@@ -1,3 +1,4 @@
+import base64
 import email
 from email.header import decode_header
 from email.message import Message as EmailMessage
@@ -17,7 +18,7 @@ except ImportError:
 
 from django.conf import settings
 from django.core.mail.message import make_msgid
-from django.core.files.base import File, ContentFile
+from django.core.files.base import ContentFile
 from django.db import models
 from django_mailbox.transports import Pop3Transport, ImapTransport,\
     MaildirTransport, MboxTransport, BabylTransport, MHTransport, \
@@ -217,17 +218,22 @@ class Mailbox(models.Model):
             new.set_payload('')
         elif msg.get_content_type() not in TEXT_STORED_MIMETYPES:
             filename = msg.get_filename()
-            extension = '.bin'
             if not filename:
                 extension = mimetypes.guess_extension(msg.get_content_type())
             else:
                 _, extension = os.path.splitext(filename)
+            if not extension:
+                extension = '.bin'
 
             attachment = MessageAttachment()
-            
+
             attachment.document.save(
                 uuid.uuid4().hex + extension,
-                ContentFile(six.BytesIO(msg.get_payload(decode=True)).getvalue())
+                ContentFile(
+                    six.BytesIO(
+                        msg.get_payload(decode=True)
+                    ).getvalue()
+                )
             )
             attachment.message = record
             for key, value in msg.items():
@@ -238,11 +244,7 @@ class Mailbox(models.Model):
             placeholder[ATTACHMENT_INTERPOLATION_HEADER] = str(attachment.pk)
             new = placeholder
         else:
-            for header, value in msg.items():
-                new[header] = value
-            new.set_payload(
-                msg.get_payload()
-            )
+            new = msg
         return new
 
     def _process_message(self, message):
@@ -258,7 +260,7 @@ class Mailbox(models.Model):
             msg.to_header = message['to']
         msg.save()
         message = self._get_dehydrated_message(message, msg)
-        msg.body = message.as_string()
+        msg.set_body(message.as_string())
         if message['in-reply-to']:
             try:
                 msg.in_reply_to = Message.objects.filter(
@@ -327,6 +329,10 @@ class Message(models.Model):
     )
 
     body = models.TextField()
+    encoded = models.BooleanField(
+        default=False,
+        help_text='True if the e-mail body is Base64 encoded'
+    )
 
     processed = models.DateTimeField(
         auto_now_add=True
@@ -354,7 +360,7 @@ class Message(models.Model):
 
         """
         addresses = []
-        addresses = self.to_addresses +  self.from_address
+        addresses = self.to_addresses + self.from_address
         return addresses
 
     @property
@@ -414,7 +420,8 @@ class Message(models.Model):
                     charset = part.get_content_charset()
                     this_part = part.get_payload(decode=True)
                     if charset:
-                        this_part = this_part.decode(charset)
+                        this_part = this_part.decode(charset, 'replace')
+
                     body += this_part
             return body
 
@@ -477,27 +484,25 @@ class Message(models.Model):
             )
         return new
 
-    def get_email_object(self):
-        """ Returns an `email.message.Message` instance for this message.
+    def get_body(self):
+        if self.encoded:
+            return base64.b64decode(self.body.encode('ascii'))
+        return self.body.encode('utf-8')
 
-        Note: Python 2's version of email.message.fromstring on some platforms
-        requires bytes and will incorrectly create an e-mail object if a
-        unicode object is passed-in.
-
-        Note that in reality 7-bit ascii *should* be an acceptable encoding
-        here per RFC-2822 (2.1), but given that `message_from_string` really
-        only cares that bytes are incoming rather than a unicode object, and
-        django will dutifully, if you were to edit the message in code or
-        via the admin, allow you to store unicode characters, returning UTF-8
-        encoded bytes is probably the safest answer.
-
-        """
-        body = self.body
-        if sys.version_info < (3, 0):
+    def set_body(self, body):
+        if sys.version_info >= (3, 0):
             body = body.encode('utf-8')
-        return self._rehydrate(
-            email.message_from_string(body)
-        )
+        self.encoded = True
+        self.body = base64.b64encode(body).decode('ascii')
+
+    def get_email_object(self):
+        """ Returns an `email.message.Message` instance for this message."""
+        body = self.get_body()
+        if sys.version_info < (3, 0):
+            flat = email.message_from_string(body)
+        else:
+            flat = email.message_from_bytes(body)
+        return self._rehydrate(flat)
 
     def delete(self, *args, **kwargs):
         for attachment in self.attachments.all():
